#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <sys/wait.h>
#include <string.h>

#define checkError(expr,userMsg) \
    do { \
        if (!(expr)) { \
            perror(userMsg);\
            exit(EXIT_FAILURE); \
        } \
    } while (0)

#define RD_END   (0)
#define WR_END   (1)
#define BUF_SIZE    (1024)

int main(int argc, char** argv) {
    
    checkError(argc == 2, "args");

    int cld2par[2];
    checkError(pipe(cld2par) != -1, "...");

    pid_t childPid = fork();
    checkError(childPid != -1, "...");

    if (childPid == 0) {

        close(cld2par[RD_END]);

        checkError(dup2(cld2par[WR_END], STDOUT_FILENO) != -1, "dup2");
        checkError(execlp("ls", "ls", "-l", argv[1], NULL) != -1, "execlp");
    }
    close(cld2par[WR_END]);

    int status = 0;
    checkError(wait(&status) != -1, "wait");

    if (WIFEXITED(status) && (WEXITSTATUS(status) == EXIT_SUCCESS)) {

        int readBytes = 0;
        char buffer[BUF_SIZE];
        while ((readBytes = read(cld2par[RD_END], buffer, BUF_SIZE-1)) > 0) {

            checkError(write(STDOUT_FILENO, buffer, readBytes) != -1, "...");
        }
        checkError(readBytes != -1, "...");
    }
    else {

        printf("Neuspeh\n");
    }

    close(cld2par[RD_END]);

    exit(EXIT_SUCCESS);
}