#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <sys/wait.h>
#include <string.h>
#include <time.h>

#include <errno.h>
#include <sys/mman.h>

#define checkError(expr,userMsg) \
    do { \
        if (!(expr)) { \
            perror(userMsg);\
            exit(EXIT_FAILURE); \
        } \
    } while (0)

#define RD_END   (0)
#define WR_END   (1)
#define BUF_SIZE    (1024)

void* getShmBlock(char* fpath, unsigned* n){

    int shmFd = shm_open(fpath, O_RDWR | O_CREAT, 0600);
    checkError(shmFd != -1, "shm_open");

    struct stat fInfo;
    checkError(fstat(shmFd, &fInfo) != -1, "fstat"); 
    *n = fInfo.st_size;

    void* adr = mmap(NULL, *n, PROT_READ|PROT_WRITE, MAP_SHARED, shmFd, 0);
    checkError(adr != MAP_FAILED, "mmap");

    close(shmFd);
    return adr;
}

int main(int argc, char** argv) {
    
    checkError(argc == 2, "args");

    unsigned n = 0;
    int* niz = getShmBlock(argv[1], &n);

    for (int i = 0; i < n/sizeof(int); i++)
        printf("%d\n", niz[i]);

    checkError(munmap(niz, n*sizeof(int)) != -1, "munmap");

    checkError(shm_unlink(argv[1]) != -1, "shm_unlink");
    
    exit(EXIT_SUCCESS);
}