#define _XOPEN_SOURCE 700
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>

#include <sys/wait.h>
#include <string.h>
#include <time.h>

#include <errno.h>
#include <sys/mman.h>

#define checkError(expr,userMsg) \
    do { \
        if (!(expr)) { \
            perror(userMsg);\
            exit(EXIT_FAILURE); \
        } \
    } while (0)

#define RD_END   (0)
#define WR_END   (1)
#define BUF_SIZE    (1024)

void* createShmBlock(char* fpath, unsigned n){

    int shmFd = shm_open(fpath, O_RDWR | O_CREAT, 0600);
    checkError(shmFd != -1, "shm_open");

    checkError(ftruncate(shmFd, n) != -1, "ftruncate");

    void* adr = mmap(NULL, n, PROT_READ|PROT_WRITE, MAP_SHARED, shmFd, 0);
    checkError(adr != MAP_FAILED, "mmap");

    close(shmFd);
    return adr;
}

int main(int argc, char** argv) {
    
    checkError(argc == 2, "args");

    int n = 10;
    int* niz = createShmBlock(argv[1], n*sizeof(int));

    for (int i = 0; i < n; i++)
        niz[i] = i+1;

    checkError(munmap(niz, n*sizeof(int)) != -1, "munmap");

    
    exit(EXIT_SUCCESS);
}